/////////////////////////////////////////////////////////////////////////
//
// CFGBASE.CPP - Definition of AConfigBase, the configuration base class
//
// (c) 1997 by Stefan Ruck, Frankfurt/Main, Germany
//
/////////////////////////////////////////////////////////////////////////

#include "cfgbase.hpp"

#pragma info (none)

#include <iexcbase.hpp>

#pragma info (restore)

//**************************************************************************
// AConfigBase :: AConfigBase - Constructor of AConfigBase                 *
//**************************************************************************

AConfigBase :: AConfigBase (const char *  pszSuffix)
                            : m_strFullFilename (),
                              m_strPureFilename (),
                              m_strPathname (),
                              m_ret (NO_ERROR)
{
  AFile file;
  IString strFile;

  filenameFromAppDir (pszSuffix);             // build the filename from the app directory

  if(tryOpenFile (file, strFile))             // the ini data can be saved in the app directory
     return;

  if(!filenameFromIniDir ())                  // == NO_ERROR
     {
      if(tryOpenFile (file, strFile))         // the ini data can be saved in the ini directory
         return;
     }

  switch((m_ret = filenameFromCurrDir ()))    // build the filename from the current directory
         {
          case NO_ERROR:
               if(tryOpenFile (file, strFile)) // the ini data can be saved in the current directory
                  return;
               break;
          case ERROR_NOT_DOS_DISK:
          case ERROR_DRIVE_LOCKED:
               break;
          default:
               ITHROW (IAccessError ("DosQueryModuleName", m_ret, IException :: unrecoverable));     // something unexpected has gone wrong
         }

  filenameFromSystemDir ();                    // build the filename from the system directory

  if(!tryOpenFile (file, strFile))             // the ini data can't be saved in the system directory
     ITHROW (IAccessError("DosOpen", m_ret, IException :: unrecoverable));     // something unexpected has gone wrong
}

//**************************************************************************
// AConfigBase :: ~AConfigBase - Destructor of AConfigBase                 *
//**************************************************************************

AConfigBase :: ~AConfigBase ()

{
}

//************************************************************************************
// AConfigBase :: filenameFromAppDir - builds the filename from the app's directory  *
//                                                                                   *
// throws IAccessError                                                               *
//************************************************************************************

AConfigBase& AConfigBase :: filenameFromAppDir (const char * pszSuffix)

{
  if(m_strFullFilename. length ())  // the filename is not empty, so it was determinated already
     return (*this);

  PPIB pPib = NULL;
  PTIB pTib = NULL;

  DosGetInfoBlocks (&pTib, &pPib);  // get the info blocks

  char pszFilename [CCHMAXPATH];

  m_ret = DosQueryModuleName (pPib -> pib_hmte, sizeof (pszFilename), pszFilename);

  if(m_ret != NO_ERROR)
     ITHROW (IAccessError ("DosQueryModuleName", m_ret, IException :: unrecoverable));

  m_strFullFilename = pszFilename;  // keep the module name

  m_strFullFilename. stripTrailing (".EXE");  // strip the exe suffix

  m_strFullFilename += ".";

  m_strFullFilename += pszSuffix;

  m_strPureFilename = m_strFullFilename. subString (m_strFullFilename. lastIndexOf ('\\'));  // save the pure filename (without directory)

  buildPathname ();

  return (*this);
}

//***************************************************************************************
// AConfigBase :: filenameFromIniDir - builds the filename from the USER_INI directory  *
//***************************************************************************************

APIRET AConfigBase :: filenameFromIniDir ()

{
  PSZ pszPath;

  m_ret = DosScanEnv ("USER_INI", (PCSZ *) &pszPath);            // scan for the value of USER_INI

  if(m_ret != NO_ERROR)
     return (m_ret);

  m_strFullFilename = pszPath;

  m_strFullFilename. remove (m_strFullFilename. lastIndexOf ('\\'));  // remove the '\os2.ini' text

  m_strFullFilename += m_strPureFilename;

  buildPathname ();

  return (m_ret);
}

//***************************************************************************************
// AConfigBase :: filenameFromCurrDir - builds the filename from the current directory  *
//***************************************************************************************

APIRET AConfigBase :: filenameFromCurrDir ()

{
  ULONG ulDiskNum = 0,
        ulLogical = 0;

  DosQueryCurrentDisk (&ulDiskNum, &ulLogical);               // query current disk

  char szCurrDir [CCHMAXPATH + 1];
  ULONG ulSize = CCHMAXPATH;

  m_ret = DosQueryCurrentDir (ulDiskNum, szCurrDir, &ulSize); // query current directory

  if(m_ret != NO_ERROR)
     return (m_ret);

  m_strFullFilename = (szCurrDir [0] ? "A:\\" : "A:");  // avoid a double \\ in pathname if curr dir is root

  m_strFullFilename += szCurrDir + m_strPureFilename;

  m_strFullFilename [0] = ulDiskNum + 'A' - 1; // convert the disk number to a character, 1 == drive A

  buildPathname ();

  return (m_ret);
}

//***************************************************************************************
// AConfigBase :: filenameFromSystemDir - builds the filename from the system directory *
//                                                                                      *
// throws IAccessError                                                                  *
//***************************************************************************************

AConfigBase& AConfigBase :: filenameFromSystemDir ()

{
  ULONG ulDiskNum = 0;

  m_ret = DosQuerySysInfo (QSV_BOOT_DRIVE, QSV_BOOT_DRIVE, &ulDiskNum, sizeof (ulDiskNum));

  if(m_ret != NO_ERROR)
     ITHROW (IAccessError("DosQuerySysInfo", m_ret, IException :: unrecoverable));

  m_strFullFilename = "A:\\OS2";

  m_strFullFilename += m_strPureFilename;

  m_strFullFilename [0] = ulDiskNum + 'A' - 1; // convert the disk number to a character, 1 == drive A

  buildPathname ();

  return (*this);
}

//***************************************************************************
// AConfigBase :: buildPathname - build the pathname from m_strFullFilename *
//***************************************************************************

AConfigBase& AConfigBase :: buildPathname ()

{
  m_strPathname = m_strFullFilename;

  m_strPathname. remove (m_strPathname. lastIndexOf ('\\'));  // remove the filename from path

  return (*this);
}

//**************************************************************************
// AConfigBase :: tryOpenFile - tries to open the ini file                 *
//                                                                         *
// throws IAccessError                                                     *
//**************************************************************************

Boolean AConfigBase :: tryOpenFile (AFile& rFile, IString& rstrFile)

{
  rstrFile = m_strPathname + "\\~tryopen.tmp";

  switch((m_ret = rFile. open (rstrFile, 0L, FILE_NORMAL, OPEN_ACTION_CREATE_IF_NEW | OPEN_ACTION_REPLACE_IF_EXISTS,
                               OPEN_FLAGS_FAIL_ON_ERROR | OPEN_ACCESS_READWRITE | OPEN_SHARE_DENYREADWRITE)))
         {
          case NO_ERROR:                      // the filename is ok
               rFile. close ();
               if (DosDelete (rstrFile))       // can't delete the testfile
                   ITHROW (IAccessError("DosDelete", m_ret, IException :: recoverable));
               return (true);
          case ERROR_WRITE_PROTECT:           // the ini data can't be saved using the filename
          case ERROR_OPEN_FAILED:
               return (false);
          default:
               {
                IString error(rstrFile);
                error += " DosOpen";
                ITHROW (IAccessError(error, m_ret, IException :: unrecoverable));  // something unexpected has gone wrong
               }
         }

  return (false);
}
